function f=trap2mf(x,params)
%  TRAP2MF Double Trapezoidal membership function.
%     TRAP2MF(X, PARAMS) returns a matrix which is the double trapezoidal
%     membership function evaluated at X. PARAMS = [A1 B1 C1 D1 A2 B2 C2 D2]
%     is a 8-element vector that determines the break points of this membership
%     function.
%     We require that A1 <= B1 <= C1 <= D1 <= A2 <= B2 <= C2 <= D2.
%  
%     For example:
%  
%         x = (0:0.1:10)';
%         y = trap2mf(x, [1 2 3 4 5 6 7 8]);
%         plot(x, y);
%  
%     See also constmf, dsigmf, evalmf, gauss2mf, gaussmf, gbellmf, mf2mf, nomf
%     pimf, psigmf, sigmf, smf, trimf, trapmf, zmf.

if nargin ~= 2
    error('Two arguments are required by the trapezoidal MF.');
elseif length(params) < 8
    error('The double trapezoidal MF needs at least eigth parameters.');
end

index = find((params(1:7) == params(2:8)));
params(index)=params(index+1)-eps;

f = ones(size(x))*eps;
a1 = params(1);
b1 = params(2);
c1 = params(3);
d1 = params(4);
a2 = params(5);
b2 = params(6);
c2 = params(7);
d2 = params(8);

if any((params(1:7)-params(2:8))>0)
    error('Illegal parameter condition: a1 <= b1 <= c1 <= d1 <= a2 <= b2 <= c2 <= d2');
end

index = find((x>=b1 & x<=c1) | (x>=b2 & x<=c2));
if ~isempty(index)
    f(index) = ones(size(index));
end
index = find(x>a1 & x<b1);
if ~isempty(index),
    f(index)=(x(index)-a1)/(b1-a1);
end
index = find(x>c1 & x<d1);
if ~isempty(index),
    f(index)=(d1-x(index))/(d1-c1);
end
index = find(x>a2 & x<b2);
if ~isempty(index),
    f(index)=(x(index)-a2)/(b2-a2);
end
index = find(x>c2 & x<d2);
if ~isempty(index),
    f(index)=(d2-x(index))/(d2-c2);
end
